@@ -7,6 +7,14 @@
 #undef HAVE_LIBSM
 #undef HAVE_PROGRAM_INVOCATION_SHORT_NAME
 #undef HAVE_PROGRAM_INVOCATION_NAME
+#undef HAVE_BONOBO
+
+#undef GNOMECC_ICONS_DIR
+#undef GNOMECC_GLADE_DIR
+#undef GNOMECC_PIXMAPS_DIR
+#undef GNOMECC_DEFAULTS_DIR
+#undef GNOMECC_SCREENSAVERS_DIR
+
 #undef PACKAGE
 #undef VERSION
 
@@ -21,5 +29,5 @@
 
 #undef WITH_SYMBOL_UNDERSCORE
 
-#undef GNOME_ICONDIR
-
+#undef HAVE_PUTENV
+#undef HAVE_SETENV
