@@ -91,7 +91,7 @@ impl_ConfigArchiver_Archive_createLocation (PortableServer_Servant         serva
 	loc = archive_create_location (ARCHIVE_FROM_SERVANT (servant), locid, label,
 				       LOCATION (bonobo_object_from_servant (parent_ref->servant)));
 
-	return bonobo_object_dup_ref (BONOBO_OBJREF (loc), ev);
+	return CORBA_Object_duplicate (BONOBO_OBJREF (loc), ev);
 }
 
 static ConfigArchiver_LocationSeq *
@@ -699,7 +699,9 @@ archive_get_backend_list (Archive *archive)
 static gint
 foreach_build_list_cb (gchar *key, Location *value, GList **node) 
 {
-	*node = g_list_prepend (*node, value);
+	if (!location_is_deleted (value))
+		*node = g_list_prepend (*node, value);
+
 	return 0;
 }
 
@@ -746,6 +748,7 @@ load_all_locations (Archive *archive)
 	DIR *archive_dir;
 	struct dirent entry, *entryp;
 	gchar *filename;
+	Location *location;
 
 	archive_dir = opendir (archive->prefix);
 
@@ -768,8 +771,11 @@ load_all_locations (Archive *archive)
 		{
 			filename = g_concat_dir_and_file (archive->prefix,
 							  entry.d_name);
-			if (g_file_test (filename, G_FILE_TEST_ISDIR))
-				archive_get_location (archive, entry.d_name);
+			if (g_file_test (filename, G_FILE_TEST_ISDIR)) {
+				location = archive_get_location (archive, entry.d_name);
+				if (location_is_deleted (location))
+					bonobo_object_unref (BONOBO_OBJECT (location));
+			}
 		}
 	}
 }
