@@ -22,6 +22,8 @@
 
 BEGIN_GNOME_DECLS
 
+#define EX_SET_NOT_FOUND(ev) bonobo_exception_set (ev, ex_Bonobo_Moniker_InterfaceNotFound)
+
 #define BONOBO_CONFIG_ARCHIVER_TYPE        (bonobo_config_archiver_get_type ())
 #define BONOBO_CONFIG_ARCHIVER(o)	   (GTK_CHECK_CAST ((o), BONOBO_CONFIG_ARCHIVER_TYPE, BonoboConfigArchiver))
 #define BONOBO_CONFIG_ARCHIVER_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), BONOBO_CONFIG_ARCHIVER_TYPE, BonoboConfigArchiverClass))
@@ -48,21 +50,24 @@ typedef struct {
 typedef struct _BonoboConfigArchiver        BonoboConfigArchiver;
 
 struct _BonoboConfigArchiver {
-	BonoboConfigDatabase     base;
+	BonoboConfigDatabase           base;
 
-	char                    *filename;
-	FILE                    *fp;
-	xmlDocPtr                doc;
-	DirData                 *dir;
-	guint                    time_id;
+	char                          *filename;
+	FILE                          *fp;
+	xmlDocPtr                      doc;
+	DirData                       *dir;
+	guint                          time_id;
 
-	ConfigArchiver_Archive   archive;
-	ConfigArchiver_Location  location;
-	gchar                   *backend_id;
-	gchar                   *real_name;
+	ConfigArchiver_Archive         archive;
+	ConfigArchiver_Location        location;
+	gchar                         *backend_id;
+	gchar                         *moniker;
 
-	BonoboEventSource       *es;
-	BonoboPropertyBag       *pb;
+	BonoboEventSource             *es;
+	BonoboPropertyBag             *pb;
+
+	Bonobo_EventSource_ListenerId  listener_id;
+	gboolean                       is_up_to_date;
 };
 
 typedef struct {
@@ -76,8 +81,7 @@ bonobo_config_archiver_get_type  (void);
 Bonobo_ConfigDatabase
 bonobo_config_archiver_new (Bonobo_Moniker               parent,
 			    const Bonobo_ResolveOptions *options,
-			    const char                  *backend_id,
-			    const char                  *location_id,
+			    const char                  *moniker,
 			    CORBA_Environment           *ev);
 
 END_GNOME_DECLS
