@@ -39,8 +39,6 @@ bonobo_config_set_filename (Bonobo_ConfigDatabase db,
 			    const char *value,
 			    CORBA_Environment *opt_ev);
 
-static Applier *applier = NULL;
-
 /* Popt option for compat reasons */
 static gchar *background_image = NULL;
 
@@ -53,23 +51,30 @@ const struct poptOption options [] = {
 static void
 apply_settings (Bonobo_ConfigDatabase db)
 {
-	Preferences *prefs;
-	CORBA_Environment ev;
+	GtkObject         *prefs;
+	Applier           *applier;
+	CORBA_Environment  ev;
 	
 	CORBA_exception_init (&ev);
-	if (!applier)
-		applier = APPLIER (applier_new ());
 
-	/* HAckity hackty */
-	if (background_image)
-	{
+	applier = APPLIER (applier_new (APPLIER_ROOT));
+
+	/* Hackity hackty */
+	if (background_image != NULL) {
 		bonobo_config_set_filename (db, "/main/wallpaper_filename", background_image, NULL);
 		Bonobo_ConfigDatabase_sync (db, &ev);
 	}
 
-	prefs = PREFERENCES (preferences_new_from_bonobo_db (db, &ev));
-	applier_apply_prefs (applier, prefs, TRUE, FALSE);
-	gtk_object_destroy (GTK_OBJECT (prefs));
+	prefs = preferences_new_from_bonobo_db (db, &ev);
+
+	if (BONOBO_EX (&ev) || prefs == NULL) {
+		g_critical ("Could not retrieve configuration from database (%s)", ev._repo_id);
+	} else {
+		applier_apply_prefs (applier, PREFERENCES (prefs));
+		gtk_object_destroy (GTK_OBJECT (prefs));
+	}
+
+	gtk_object_destroy (GTK_OBJECT (applier));
 
 	CORBA_exception_free (&ev);
 }
@@ -130,10 +135,10 @@ copy_color_from_legacy (Bonobo_ConfigDatabase db,
 }
 
 static void
-bonobo_config_set_filename (Bonobo_ConfigDatabase db,
-			    const char *key,
-			    const char *value,
-			    CORBA_Environment *opt_ev)
+bonobo_config_set_filename (Bonobo_ConfigDatabase  db,
+			    const char            *key,
+			    const char            *value,
+			    CORBA_Environment     *opt_ev)
 {
 	CORBA_any *any;
 	
@@ -148,11 +153,21 @@ get_legacy_settings (Bonobo_ConfigDatabase db)
 {
 	gboolean val_boolean, def;
 	gchar *val_string, *val_filename;
-	int val_ulong, val_long;
+	int val_ulong = -1, val_long = -1;
+
+	static const int wallpaper_types[] = { 0, 1, 3, 2 };
 
 	COPY_FROM_LEGACY (boolean, "/main/enabled", bool, "/Background/Default/Enabled=true");
-	COPY_FROM_LEGACY (filename, "/main/wallpaper_filename", string, "/Background/Default/wallpaper=none");
-	COPY_FROM_LEGACY (ulong, "/main/wallpaper_type", int, "/Background/Default/wallpaperAlign=0");
+	COPY_FROM_LEGACY (filename, "/main/wallpaper_filename", string, "/Background/Default/wallpaper=(none)");
+
+	if (val_filename != NULL && strcmp (val_filename, "(none)"))
+		bonobo_config_set_boolean (db, "/main/wallpaper_enabled", TRUE, NULL);
+	else
+		bonobo_config_set_boolean (db, "/main/wallpaper_enabled", FALSE, NULL);
+
+	val_ulong = gnome_config_get_int ("/Background/Default/wallpaperAlign=0");
+	bonobo_config_set_ulong (db, "/main/wallpaper_type", wallpaper_types[val_ulong], NULL);
+
 	copy_color_from_legacy (db, "/main/color1", "/Background/Default/color1");
 	copy_color_from_legacy (db, "/main/color2", "/Background/Default/color2");
 
@@ -177,51 +192,91 @@ get_legacy_settings (Bonobo_ConfigDatabase db)
 
 	val_boolean = gnome_config_get_bool_with_default ("/Background/Default/adjustOpacity=true", &def);
 
-	if (!def && val_boolean) {
+	if (!def && val_boolean)
 		COPY_FROM_LEGACY (long, "/main/opacity", int, "/Background/Default/opacity=100");
-	}
 }
 
 static void
-property_change_cb (BonoboListener *listener,
-		    char *event_name,
-		    CORBA_any *any,
-		    CORBA_Environment *ev,
-		    Bonobo_PropertyBag pb)
+property_change_cb (BonoboListener     *listener,
+		    char               *event_name,
+		    CORBA_any          *any,
+		    CORBA_Environment  *ev,
+		    Preferences        *prefs)
 {
-	Preferences *prefs;
-
-	prefs = PREFERENCES (preferences_new_from_bonobo_pbag (pb, ev));
-	applier_apply_prefs (applier, prefs, FALSE, TRUE);
-	gtk_object_destroy (GTK_OBJECT (prefs));
-}
-
-static void
-realize_cb (GtkWidget *widget, Bonobo_PropertyBag bag)
-{
-	CORBA_Environment ev;
-	Preferences *prefs;
+	GladeXML *dialog;
+	Applier  *applier;
 	
-	CORBA_exception_init (&ev);
-       	prefs = PREFERENCES (preferences_new_from_bonobo_pbag (bag, &ev));
-	applier_apply_prefs (applier, prefs, FALSE, TRUE);
-	gtk_object_destroy (GTK_OBJECT (prefs));
-	CORBA_exception_free (&ev);
+	g_return_if_fail (prefs != NULL);
+	g_return_if_fail (IS_PREFERENCES (prefs));
+
+	if (GTK_OBJECT_DESTROYED (prefs))
+		return;
+
+	dialog = gtk_object_get_data (GTK_OBJECT (prefs), "glade-data");
+
+	preferences_apply_event (prefs, event_name, any);
+	applier = gtk_object_get_data (GTK_OBJECT (WID ("prefs_widget")), "applier");
+	applier_apply_prefs (applier, prefs);
+
+	if (!strcmp (event_name, "Bonobo/Property:change:wallpaper_type")
+	    || !strcmp (event_name, "Bonobo/Property:change:wallpaper_filename")
+	    || !strcmp (event_name, "Bonobo/Property:change:wallpaper_enabled"))
+		gtk_widget_set_sensitive
+			(WID ("color_frame"), applier_render_color_p (applier, prefs));
 }
 
-#define CUSTOM_CREATE_PEDITOR(type, corba_type, key, widget)                                              \
+static gboolean
+real_realize_cb (Preferences *prefs) 
+{
+	GladeXML *dialog;
+	Applier  *applier;
+
+	g_return_val_if_fail (prefs != NULL, TRUE);
+	g_return_val_if_fail (IS_PREFERENCES (prefs), TRUE);
+
+	if (GTK_OBJECT_DESTROYED (prefs))
+		return FALSE;
+
+	dialog = gtk_object_get_data (GTK_OBJECT (prefs), "glade-data");
+	applier = gtk_object_get_data (GTK_OBJECT (WID ("prefs_widget")), "applier");
+
+	applier_apply_prefs (applier, prefs);
+
+	gtk_widget_set_sensitive (WID ("color_frame"), applier_render_color_p (applier, prefs));
+
+	return FALSE;
+}
+
+static gboolean
+realize_2_cb (Preferences *prefs) 
+{
+	gtk_idle_add ((GtkFunction) real_realize_cb, prefs);
+	return FALSE;
+}
+
+static void
+realize_cb (GtkWidget *widget, Preferences *prefs)
+{
+	gtk_timeout_add (100, (GtkFunction) realize_2_cb, prefs);
+}
+
+#define CUSTOM_CREATE_PEDITOR(type, corba_type, key, widget)                           \
         {                                                                              \
 		BonoboPEditor *ed = BONOBO_PEDITOR                                     \
 			(bonobo_peditor_##type##_construct (WID (widget)));            \
-		bonobo_peditor_set_property (ed, bag, key, TC_##corba_type, NULL);           \
+		bonobo_peditor_set_property (ed, bag, key, TC_##corba_type, NULL);     \
 	}
 
-
 static void
 setup_dialog (GtkWidget *widget, Bonobo_PropertyBag bag)
 {
-	GladeXML *dialog;
-	Applier *applier;
+	GladeXML                      *dialog;
+	Applier                       *applier;
+	GtkObject                     *prefs;
+
+	CORBA_Environment              ev;
+
+	CORBA_exception_init (&ev);
 
 	dialog = gtk_object_get_data (GTK_OBJECT (widget), "glade-data");
 	CUSTOM_CREATE_PEDITOR (option_menu, ulong, "orientation", "color_option");	
@@ -233,15 +288,37 @@ setup_dialog (GtkWidget *widget, Bonobo_PropertyBag bag)
 	CUSTOM_CREATE_PEDITOR (option_menu, ulong, "wallpaper_type", "image_option");	
 	CUSTOM_CREATE_PEDITOR (int_range, long, "opacity", "opacity_spin");
 
+	CREATE_PEDITOR (boolean, "wallpaper_enabled", "picture_enabled_check");
+
+	bonobo_peditor_set_guard (WID ("picture_frame"), bag, "wallpaper_enabled");
+			
+	/* Disable opacity controls */
 	gtk_widget_hide (WID ("opacity_spin"));
 	gtk_widget_hide (WID ("opacity_label"));
 
 	bonobo_property_bag_client_set_value_gboolean (bag, "enabled", TRUE, NULL);
+
+	prefs = preferences_new_from_bonobo_pbag (bag, &ev);
+
+	if (BONOBO_EX (&ev) || prefs == NULL)
+		g_error ("Could not retrieve configuration from property bag (%s)", ev._repo_id);
+
+	gtk_object_set_data (prefs, "glade-data", dialog);
+
 	bonobo_event_source_client_add_listener (bag, (BonoboListenerCallbackFn) property_change_cb,
-						 NULL, NULL, bag);
+						 NULL, NULL, prefs);
 
 	applier = gtk_object_get_data (GTK_OBJECT (widget), "applier");
-	gtk_signal_connect_after (GTK_OBJECT (applier_get_preview_widget (applier)), "realize", realize_cb, bag);
+
+	if (GTK_WIDGET_REALIZED (applier_get_preview_widget (applier)))
+		applier_apply_prefs (applier, PREFERENCES (prefs));
+	else
+		gtk_signal_connect_after (GTK_OBJECT (applier_get_preview_widget (applier)), "realize", realize_cb, prefs);
+
+	gtk_signal_connect_object (GTK_OBJECT (widget), "destroy",
+				   GTK_SIGNAL_FUNC (gtk_object_destroy), prefs);
+
+	CORBA_exception_free (&ev);
 }
 
 static GtkWidget*
@@ -249,27 +326,23 @@ create_dialog (void)
 {
 	GtkWidget *holder;
 	GtkWidget *widget;
-	GladeXML *dialog;
+	GladeXML  *dialog;
+	Applier   *applier;
 
 	dialog = glade_xml_new (GNOMECC_GLADE_DIR "/background-properties.glade", "prefs_widget");
 	widget = glade_xml_get_widget (dialog, "prefs_widget");
 	gtk_object_set_data (GTK_OBJECT (widget), "glade-data", dialog);
 
-	applier = APPLIER (applier_new ());
+	applier = APPLIER (applier_new (APPLIER_PREVIEW));
 	gtk_object_set_data (GTK_OBJECT (widget), "applier", applier);
+	gtk_signal_connect_object (GTK_OBJECT (widget), "destroy", GTK_SIGNAL_FUNC (gtk_object_destroy), GTK_OBJECT (applier));
 
 	/* Minor GUI addition */
 	holder = WID ("preview_holder");
-	gtk_box_pack_start (GTK_BOX (holder),
-			    applier_get_preview_widget (applier),
-			    TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (holder), applier_get_preview_widget (applier), TRUE, TRUE, 0);
 	gtk_widget_show_all (holder);
 
-	gnome_entry_append_history (GNOME_ENTRY (gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (WID ("image_fileentry")))), 0, "(none)");
-
-	gtk_signal_connect_object (GTK_OBJECT (widget), "destroy",
-				   GTK_SIGNAL_FUNC (gtk_object_destroy),
-			   	   GTK_OBJECT (dialog));
+	gtk_signal_connect_object (GTK_OBJECT (widget), "destroy", GTK_SIGNAL_FUNC (gtk_object_destroy), GTK_OBJECT (dialog));
 
 	return widget;
 }
@@ -284,7 +357,5 @@ main (int argc, char **argv)
 
 	capplet_init (argc, argv, legacy_files, apply_settings, create_dialog, setup_dialog, get_legacy_settings);
 
-	gnome_window_icon_set_default_from_file
-		(GNOMECC_ICONS_DIR"/gnome-ccbackground.png");
 	return 0;
 }
