@@ -77,8 +77,11 @@ apply_cb (BonoboPropertyControl *pc, Bonobo_PropertyControl_Action action)
  */
 
 static void
-changed_cb (BonoboListener *listener, gchar *event_name, CORBA_any *any,
-	    CORBA_Environment *ev, Bonobo_ConfigDatabase db) 
+changed_cb (BonoboListener        *listener,
+	    gchar                 *event_name,
+	    CORBA_any             *any,
+	    CORBA_Environment     *ev,
+	    Bonobo_ConfigDatabase  db) 
 {
 	if (apply_settings_cb != NULL)
 		apply_settings_cb (db);
@@ -117,14 +120,18 @@ pf_destroy_cb (BonoboPropertyFrame *pf, Bonobo_ConfigDatabase db)
  */
 
 static void
-set_moniker_cb (BonoboPropertyBag *bag, BonoboArg *arg, guint arg_id,
-		CORBA_Environment *ev, BonoboControl *control) 
+set_moniker_cb (BonoboPropertyBag *bag, 
+		BonoboArg         *arg,
+		guint              arg_id,
+		CORBA_Environment *ev,
+		BonoboControl     *control) 
 {
-	gchar *moniker;
-	gchar *full_moniker;
-	BonoboPropertyFrame *pf;
-	Bonobo_PropertyBag proxy;
-	Bonobo_ConfigDatabase db;
+	gchar                 *moniker;
+	gchar                 *full_moniker;
+	BonoboPropertyFrame   *pf;
+	Bonobo_PropertyBag     proxy;
+	Bonobo_ConfigDatabase  db;
+	gboolean               need_setup_cb = TRUE;
 
 	if (arg_id != 1) return;
 
@@ -133,6 +140,7 @@ set_moniker_cb (BonoboPropertyBag *bag, BonoboArg *arg, guint arg_id,
 
 	pf = BONOBO_PROPERTY_FRAME (bonobo_control_get_widget (control));
 	bonobo_property_frame_set_moniker (pf, full_moniker);
+	g_free (full_moniker);
 
 	if (pf->proxy->bag == CORBA_OBJECT_NIL) {
 		bonobo_exception_set (ev, ex_Bonobo_Property_InvalidValue);
@@ -143,8 +151,12 @@ set_moniker_cb (BonoboPropertyBag *bag, BonoboArg *arg, guint arg_id,
 
 	db = gtk_object_get_data (GTK_OBJECT (pf), "config-database");
 
-	if (db != CORBA_OBJECT_NIL)
+	if (db != CORBA_OBJECT_NIL) {
+		gtk_signal_disconnect_by_func (GTK_OBJECT (pf),
+					       GTK_SIGNAL_FUNC (pf_destroy_cb), db);
 		bonobo_object_release_unref (db, ev);
+		need_setup_cb = FALSE;
+	}
 
 	db = bonobo_get_object (moniker, "IDL:Bonobo/ConfigDatabase:1.0", ev);
 
@@ -155,7 +167,7 @@ set_moniker_cb (BonoboPropertyBag *bag, BonoboArg *arg, guint arg_id,
 	gtk_signal_connect (GTK_OBJECT (pf), "destroy",
 			    GTK_SIGNAL_FUNC (pf_destroy_cb), db);
 
-	if (setup_cb != NULL)
+	if (setup_cb != NULL && need_setup_cb)
 		setup_cb (GTK_BIN (pf)->child, proxy);
 }
 
@@ -310,6 +322,7 @@ get_factory_name (const gchar *binary)
 	tmp = strrchr (s, '/');
 	if (tmp == NULL) tmp = s;
 	else tmp++;
+	if ((tmp1 = strstr (tmp, "-control")) != NULL) *tmp1 = '\0';
 	if ((tmp1 = strstr (tmp, "-capplet")) != NULL) *tmp1 = '\0';
 	while ((tmp1 = strchr (tmp, '-')) != NULL) *tmp1 = '_';
 
@@ -332,6 +345,7 @@ get_default_moniker (const gchar *binary)
 	tmp = strrchr (s, '/');
 	if (tmp == NULL) tmp = s;
 	else tmp++;
+	if ((tmp1 = strstr (tmp, "-control")) != NULL) *tmp1 = '\0';
 	if ((tmp1 = strstr (tmp, "-capplet")) != NULL) *tmp1 = '\0';
 
 	res = g_strconcat ("archive:user-archive#archiverdb:", tmp, NULL);
@@ -353,6 +367,7 @@ get_property_name (const gchar *binary)
 	tmp = strrchr (s, '/');
 	if (tmp == NULL) tmp = s;
 	else tmp++;
+	if ((tmp1 = strstr (tmp, "-control")) != NULL) *tmp1 = '\0';
 	if ((tmp1 = strstr (tmp, "-capplet")) != NULL) *tmp1 = '\0';
 
 	for (tmp1 = tmp; *tmp1 != '\0'; tmp1++) {
@@ -418,24 +433,23 @@ legacy_is_modified (Bonobo_ConfigDatabase db, const gchar *filename)
 	struct stat stbuf;
 	gchar *realfile;
 	struct tm *legacy_tm;
+	gboolean ret;
 	
 	g_return_val_if_fail (db != CORBA_OBJECT_NIL, FALSE);
 
 	CORBA_exception_init (&ev);
-	
+
 	pb = Bonobo_Unknown_queryInterface (db, "IDL:Bonobo/PropertyBag:1.0", &ev);
-	if (BONOBO_EX (&ev)) {
-		CORBA_exception_free (&ev);
+
+	if (pb == CORBA_OBJECT_NIL || BONOBO_EX (&ev))
 		return FALSE;
-	}
 
 	arg = bonobo_property_bag_client_get_value_any (pb, "last_modified", &ev);
-	if (BONOBO_EX (&ev)) {
-		CORBA_exception_free (&ev);
-		return FALSE;
-	}
 
-	bonobo_object_release_unref (pb, &ev);
+	bonobo_object_release_unref (pb, NULL);
+
+	if (arg == NULL || BONOBO_EX (&ev))
+		return FALSE;
 
 	log_val = BONOBO_ARG_GET_GENERAL (arg, TC_ulonglong, CORBA_unsigned_long_long, NULL);
 	bonobo_arg_release (arg);
@@ -449,16 +463,18 @@ legacy_is_modified (Bonobo_ConfigDatabase db, const gchar *filename)
 					NULL);
 
 	if (stat (realfile, &stbuf) != 0) {
-		CORBA_exception_free (&ev);
-		g_free (realfile);
-		return FALSE;
+		ret = FALSE;
+	} else {
+		legacy_tm = localtime (&stbuf.st_mtime);
+		legacy_val = mktime (legacy_tm);
+		ret = (legacy_val > log_val);
 	}
-	
-	CORBA_exception_free (&ev);
+
 	g_free (realfile);
-	legacy_tm = localtime (&stbuf.st_mtime);
-	legacy_val = mktime (legacy_tm);
-	return (legacy_val > log_val);
+
+	CORBA_exception_free (&ev);
+
+	return ret;
 }
 
 /* capplet_init -- see documentation in capplet-util.h
